package PseudoProxy;

import java.io.*;
import java.util.StringTokenizer;
import java.net.*;

public class HttpRequestHdr
{
  private String method=null;
  private String url=null;
  private String version=null;
  private String unrecognized=null;
  private String byteContentRangeSpec=null;
  private boolean isEntityComplete;
  private int firstBytePos;
  private int lastBytePos;
  private int entityLength;
  private int hdrFieldIndex;
  private int contentLength;
  private String hdrField[]=null;
  private boolean isLastMessage;
  private boolean isMessageEncoded;
  private boolean isToForward;
  private boolean pragmaNoCache;

  //I SEGUENTI SONO I CAMPI HEADER DI NOSTRO INTERESSE
  private static String headerOfInterest[]={"CONNECTION:","TRANSFER-ENCODING:","VIA:","HOST:","MAX-FORWARDS:","CONTENT-ENCODING:","CONTENT-LENGTH:","CONTENT-RANGE:","ACCEPT-ENCODING:","IF-MODIFIED-SINCE:","IF-NONE-MATCH:"};
  private static String CRLF="\r\n";
  private static String SP=" ";
  private static String HT="\t";

  public HttpRequestHdr()
  {
    method=new String();
    url=new String();
    version=new String();
    unrecognized=new String();
    byteContentRangeSpec=new String();
    isLastMessage=false;
    isMessageEncoded=false;
    pragmaNoCache=false;
    isEntityComplete=true;
    isToForward=true;
    hdrField=new String[headerOfInterest.length];
    for (int i=0;i<headerOfInterest.length;i++)
      hdrField[i]=new String();
    hdrFieldIndex=0;
    contentLength=0;
    firstBytePos=0;
    lastBytePos=0;
    entityLength=0;
  }
  
  public int getContentLength()
  {
    return contentLength;
  }

  public boolean getIsLastMessage()
  {
    return isLastMessage;
  }

  public void setIsLastMessage(boolean lastMess)
  {
    isLastMessage=lastMess;
  }
  
  public boolean getIsToForward()
  {
    return isToForward;
  }
  
  public boolean getIsMessageEncoded()
  {
    return isMessageEncoded;
  }
  
  public boolean getIsEntityComplete()
  {
    return isEntityComplete;
  }
  
  public int getFirstBytePos()
  {
    return firstBytePos;
  }
  
  public int getLastBytePos()
  {
    return lastBytePos;
  }

  public int getEntityLength()
  {
    return entityLength;
  }
  
  public String getMethod()
  {
    return method;
  }
  
  public String getUrl()
  {
    return url;
  }
  
  public void setUrl(String newUrl)
  {
    url=newUrl;
  }
  
  public boolean parse(InputStream in)
  {
    //BufferedReader lines;
    DataInputStream lines;
    StringTokenizer sToken;
    try{
      lines=new DataInputStream(in);
      sToken=new StringTokenizer(lines.readLine());
    }
    catch (Exception e){
      return false;
    }
    /* Request= Request-line
          *(general-header
          | request-header
          | entity-header)
          CRLF
          [message-body]
    */
    /* Request-line=Method SP 
            Request-URI SP 
            HTTP-Version CRLF
    */
    method=getToken(sToken).toUpperCase();
    url=getToken(sToken);
    version=getToken(sToken);
    String lastToken=version;
    /*
    Message-header=Field-name ":" [Field-value] CRLF
    Field-name=token
    Field-value=*(Field-content|LWS)
    LWS=[CRLF] 1*(SP|HT)
    */
    try{
      sToken=new StringTokenizer(lines.readLine());
    }
    catch(Exception e){
      return false;
    }
    lastToken=getToken(sToken);
    while (lastToken.length()!=0)
    {
      hdrFieldIndex=recognizeToken(lastToken);
      switch (hdrFieldIndex)
      {
      case 0:
        //Connection="CONNECTION"":"1#Connection-token
        String connToken="";
        if (sToken.hasMoreTokens())
        {
          connToken=sToken.nextToken();
          hdrField[hdrFieldIndex]=new String(connToken);
        }
        //"CONNECTION: CLOSE"
        if (connToken.equalsIgnoreCase("CLOSE"))
          isLastMessage=true;
        while (sToken.hasMoreTokens())
        {
          connToken=sToken.nextToken();
          hdrField[hdrFieldIndex]+=" "+connToken.toString();
          if ((connToken.equalsIgnoreCase("CLOSE"))&&!(isLastMessage))
            isLastMessage=true;
        }
        break;
      case 1:
        //Transfer-Encoding="TRANSFER-ENCODING"":" 1#Transfer-coding
        isMessageEncoded=true;
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 2:
        //Via="VIA"":" 1#(Received-protocol received-by [comment])
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        //Si inserisce in coda il protocollo ricevuto
        hdrField[hdrFieldIndex]+=", "+version;
        //Si inserisce il nome del proxy o l'host
        try 
        {
          hdrField[hdrFieldIndex]+=InetAddress.getLocalHost().getHostName();
        }
        catch (UnknownHostException uhe)
        {}
        break;
      case 3:
        //Host="HOST"":" Host [":" Port]
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 4:
        //Max-Forwards="MAX-FORWARDS"":" 1*DIGIT
        if (method.equalsIgnoreCase("TRACE"))
        {
          int maxForwards=Integer.parseInt(getToken(sToken));
          if (maxForwards>0)
          {
            maxForwards--;
            hdrField[hdrFieldIndex]+=" "+maxForwards;
          }
          else
            isToForward=false;
        }
        break;
      case 5:
        //Content-Encoding="CONTENT-ENCODING"":" 1#Content-coding
        isMessageEncoded=true;
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 6:
        //Content-Length="CONTENT-LENGTH"":" 1*DIGIT
        contentLength=Integer.parseInt(getToken(sToken));
        hdrField[hdrFieldIndex]+=" "+contentLength;
      case 7:
        //Content-Range="CONTENT-RANGE"":" Content-range-spec
        //Content-range-spec=Byte-content-range-spec
        //Byte-content-range-spec="bytes" SP First-byte-pos "-"
        //            Last-byte-pos "/" Entity-length
        //Entity-length=1*DIGIT            
        hdrField[hdrFieldIndex]+=" "+getToken(sToken)+" ";
        byteContentRangeSpec=getRemainder(sToken);
        firstBytePos=Integer.parseInt(byteContentRangeSpec.substring(0,byteContentRangeSpec.indexOf("-")));
        lastBytePos=Integer.parseInt(byteContentRangeSpec.substring(byteContentRangeSpec.indexOf("-")+1,hdrField[hdrFieldIndex].indexOf("/")));
        entityLength=Integer.parseInt(byteContentRangeSpec.substring(byteContentRangeSpec.indexOf("/")+1));
        if ((entityLength!=(lastBytePos+1))||(firstBytePos!=0))
        //isEntityComplete=false solo se non
        //viene richiesto il documento completo
        //ma solo una parte
          isEntityComplete=false;
        break;
      case 8:
        //Accept-Encoding="ACCEPT-ENCODING"":" #(Content-coding)
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 9:
        //If-Modified-Since="IF-MODIFIED-SINCE"":"HTTP-date
        getRemainder(sToken);
        break;
      case 10:
        //If-None-Match="IF-NONE-MATCH"":"("*"|1#(entity-tag))
        getRemainder(sToken);
        break;
      case -1:
        //Token non riconosciuto
        unrecognized+=lastToken+" "+getRemainder(sToken)+CRLF;
        break;
      }//Fine switch
      try{
        sToken=new StringTokenizer(lines.readLine());
      }
      catch(Exception e){
        return false;
      }
      lastToken=getToken(sToken);
    }//Fine while
    //Nel socket ora si trova il message-body
    //Se il campo Host non  presente...
    if (hdrField[3].length()<=0)
    {
      URL serverUrl;
      try{
        serverUrl=new URL(url);
      }
      catch (MalformedURLException mue){
        return false;
      }
      //...lo inseriamo
      hdrField[3]+=" "+serverUrl.getHost()+":"+serverUrl.getPort();
    }
    //Se il protocollo che arriva  inferiore all'1.1...
    if (!version.equalsIgnoreCase("HTTP/1.1"))
    {
      //...si inserisce l'header field "CONNECTION: CLOSE"
      isLastMessage=true;
      hdrField[0]+=" CLOSE";
    }
    return true;
  }//Fine parse
   
   //Ritorna il prossimo token in una stringa
   private String  getToken(StringTokenizer tk)
   {
     String str ="";
     if  (tk.hasMoreTokens())
       str =tk.nextToken();
     return str; 
   }//Fine getToken
   
   //Riconosce l'ultimo token letto
   private int recognizeToken(String tok)
   {
    int i;
    for (i=0;i<headerOfInterest.length;i++)
      if (tok.equalsIgnoreCase(headerOfInterest[i]))
        return i;
    //Token non riconosciuto      
    return -1;
   }//Fine recognizeToken
   
   //Ricompone il message header
   public String toString(boolean siteOfInterest) 
   {
     String Request; 
     if (0==method.length())
      method="GET";
     Request=method+" "+url+" HTTP/1.0"+CRLF;
     //se la richiesta  una GET...
     if (method.equalsIgnoreCase("GET"))
      //...non si accettano codifiche   
      hdrField[8]=" ";
     if (hdrField[7].length()>0)
       {
       // se  un sito di interesse
       if (siteOfInterest)
        //si richiede il documento intero
        hdrField[7]="";
       else
        //altrimenti si richiede solo il range richiesto dal client
        hdrField[7]+=byteContentRangeSpec;
       }
     for (int i=0;i<9;i++)
     {
       if (hdrField[i].length()>0)
        Request+=headerOfInterest[i]+hdrField[i]+CRLF;
     }
     if (unrecognized.length()>0)
       Request+=unrecognized;
     Request += CRLF;
     return Request;
   }//Fine toString

   String  getRemainder(StringTokenizer tk){
     String str ="";
     if  (tk.hasMoreTokens())
       str =tk.nextToken();
     while (tk.hasMoreTokens()){
       str +=" " + tk.nextToken();
     }
     return str;
   }//Fine getRemainder   

}//Fine classe HttpRequestHdr
