package PseudoProxy;

import java.io.*;
import java.util.StringTokenizer;
import java.net.*;

public class HttpResponseHdr
{
  private String statusCode=null;
  private String version=null;
  private String reasonPhrase=null;
  private String unrecognized=null;
  private int contentLength;
  private String contentType=null;
  private boolean isLastMessage;
  private boolean isMessageEncoded;
  private String hdrField[]=null;
  private int hdrFieldIndex;
  private String byteContentRangeSpec=null;
  //I SEGUENTI SONO I CAMPI HEADER DI NOSTRO INTERESSE
  private static String headerOfInterest[]={"CONNECTION:","TRANSFER-ENCODING:","VIA:","CONTENT-ENCODING:","CONTENT-TYPE:","CONTENT-LENGTH:"};
  private static String CRLF="\r\n";
  private static String SP=" ";
  private static String HT="\t";

  public HttpResponseHdr()
  {
    version=new String();
    statusCode=new String();
    reasonPhrase=new String();
    unrecognized=new String();
    byteContentRangeSpec=new String();
    contentType=new String();
    isLastMessage=false;
    isMessageEncoded=false;
    hdrField=new String[headerOfInterest.length];
    for (int i=0;i<headerOfInterest.length;i++)
      hdrField[i]=new String();
    hdrFieldIndex=0;
    contentLength=0;
  }//Fine costruttore

  public String getContentType()
  {
    return contentType;
  }
  
  public int getContentLength()
  {
    return contentLength;
  }
  
  public void setContentLength(int cl)
  {
    contentLength=cl;
  }

  public boolean getIsLastMessage()
  {
    return isLastMessage;
  }

  public boolean getIsMessageEncoded()
  {
    return isMessageEncoded;
  }
  
  public String getStatusCode()
  {
    return statusCode;
  }

  public boolean parse(InputStream in)
  {
    DataInputStream lines;
    StringTokenizer sToken;
    lines=new DataInputStream(in);
    try
    {
      sToken=new StringTokenizer(lines.readLine());    
    }
    catch (Exception e){
      return false;
    }
    /* Response=Status-line
          *(general-header
          | response-header
          | entity-header)
          CRLF
          [message-body]
    */
    /* Status-line= HTTP-Version SP 
            Status-Code SP
            Reason-Phrase CRLF
    */
    version=getToken(sToken);
    statusCode=getToken(sToken);
    reasonPhrase=getToken(sToken);
    String lastToken=version;
    /*
    Message-header=Field-name ":" [Field-value] CRLF
    Field-name=token
    Field-value=*(Field-content|LWS)
    LWS=[CRLF] 1*(SP|HT)
    */
    try
    {
      sToken=new StringTokenizer(lines.readLine());
    }
    catch(Exception e)
    {
      return false;
    }
    lastToken=getToken(sToken);
    while (lastToken.length()!=0)
    {
      hdrFieldIndex=recognizeToken(lastToken);
      switch (hdrFieldIndex)
      {
      case 0:
        //Connection="CONNECTION"":"1#Connection-token
        String connToken="";
        if (sToken.hasMoreTokens())
        {
          connToken=sToken.nextToken();
          hdrField[hdrFieldIndex]=new String(connToken);
        }
        //"CONNECTION: CLOSE"
        if (connToken.equalsIgnoreCase("CLOSE"))
          isLastMessage=true;
        while (sToken.hasMoreTokens())
        {
          connToken=sToken.nextToken();
          hdrField[hdrFieldIndex]+=" "+connToken.toString();
          if ((connToken.equalsIgnoreCase("CLOSE"))&&!(isLastMessage))
            isLastMessage=true;
        }
        break;
      case 1:
        //Transfer-Encoding="TRANSFER-ENCODING"":" 1#Transfer-coding
        isMessageEncoded=true;
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 2:
        //Via="VIA"":" 1#(Received-protocol received-by [comment])
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        //Si inserisce in coda il protocollo ricevuto
        hdrField[hdrFieldIndex]+=", "+version;
        //Si inserisce il nome del proxy o l'host
        try
        {
          InetAddress.getLocalHost().getHostName();
        }
        catch (UnknownHostException uhe)
        {}
        break;
      case 3:
        //Content-Encoding="CONTENT-ENCODING"":" 1#Content-coding
        isMessageEncoded=true;
        hdrField[hdrFieldIndex]+=" "+getRemainder(sToken);
        break;
      case 4:
        //Content-Type="CONTENT-TYPE"":" Media-Type
        //Media-Type=Type "/" Subtype *(";" Parameter)
        //Parameter= Attribute "=" Value
        contentType=getRemainder(sToken);
        hdrField[hdrFieldIndex]+=" "+contentType;
        break;
      case 5:
        //Content-Length="CONTENT-LENGTH"":" 1*DIGIT
        contentLength=Integer.parseInt(getToken(sToken));
        hdrField[hdrFieldIndex]+=" "+contentLength;
        break;
      case -1:
        //Token non riconosciuto
        unrecognized+=lastToken+" "+getRemainder(sToken)+CRLF;
        break;
      }//Fine switch
      try
      {
        sToken=new StringTokenizer(lines.readLine());
      }
      catch(Exception e)
      {
        return false;
      }
      lastToken=getToken(sToken);
    }//Fine while
    //Nel socket ora si trova il message-body
    //Se il protocollo che arriva  inferiore all'1.1...
    if (!version.equalsIgnoreCase("HTTP/1.1"))
    {
      //...si inserisce l'header field "CONNECTION: CLOSE"
      isLastMessage=true;
      hdrField[0]+=" CLOSE";
    }    
    return true;
  }//Fine parse

   //Ritorna il prossimo token in una stringa
   private String  getToken(StringTokenizer tk)
   {
     String str ="";
     if  (tk.hasMoreTokens())
       str =tk.nextToken();
     return str;
   }//Fine getToken

   //Riconosce l'ultimo token letto
   private int recognizeToken(String tok)
   {
    int i;
    for (i=0;i<headerOfInterest.length;i++)
      if (tok.equalsIgnoreCase(headerOfInterest[i]))
        return i;
    //Token non riconosciuto
    return -1;
   }//Fine recognizeToken

   //Ricompone il message header
   public String toString(boolean modified)
   {
     String Response;
     if (modified)
     {
        Response="HTTP/1.1 200 "+"OK"+CRLF;
        hdrField[5]=" "+contentLength;
     }
     else
     {
      Response="HTTP/1.1 "+statusCode+" "+reasonPhrase+CRLF;
     }
     for (int i=0;i<hdrField.length;i++)
     {
      if (hdrField[i].length()>0)
      Response+=headerOfInterest[i]+hdrField[i]+CRLF;
     }
     if (unrecognized.length()>0)
       Response+=unrecognized;
     Response += CRLF;
     return Response;
   }//Fine toString

   String  getRemainder(StringTokenizer tk){
     String str ="";
     if  (tk.hasMoreTokens())
       str =tk.nextToken();
     while (tk.hasMoreTokens()){
       str +=" " + tk.nextToken();
     }
     return str;
   }//Fine getRemainder

}//Fine classe HttpResponseHdr
