package PseudoProxy;

import java.net.*;
import java.io.*;
import java.util.*;

public class Proxy extends Thread
{
  //Member variables
  Socket clientSocket=null;
  Socket serverSocket=null;
  Config config=null;
  String localHostName=null;
  String localHostIP=null;
  String adminPath=null;
  boolean isLastMessage;
  boolean isServerSocketAlive;
  

  public Proxy(Socket cSocket, Config cnfg)
  {
    clientSocket=cSocket;
    config=cnfg;
    localHostName=config.getLocalHost();
    localHostIP=config.getLocalIP();
    adminPath=config.getAdminPath();
    isLastMessage=false;
    isServerSocketAlive=false;
  }// Fine costruttore
  
  public void run()
  {
    boolean htmlMonitor=false;
    int hostIndex;
    int docIndex;
    int linkIndex;
    int bufferLength;
    byte AppByte[];
    String serverName=new String();
    String query=new String();
    InputStream clientIn;
    OutputStream clientOut;
    InputStream serverIn;
    OutputStream serverOut;
    DataOutputStream serverDataOut;
    DataOutputStream clientDataOut;
    BufferedInputStream serverBufferIn;
    URL url;
    HtmlResponseBody body=null;
    HttpRequestHdr request=null;
    HttpResponseHdr response=null;
    try
    {
      config.openThread();
      clientIn=clientSocket.getInputStream();
      clientOut=clientSocket.getOutputStream();
      // Trattamento della connessione....
      do
      {
        //Parsing dell'header della richiesta del client
        request=new HttpRequestHdr();
        request.parse(clientIn);
        url=new URL(request.getUrl());
        isLastMessage=request.getIsLastMessage();
        //se  una richiesta dell'amministratore del proxy...
        if (url.getFile().equalsIgnoreCase("/admin")&&
          (url.getHost().equalsIgnoreCase(localHostName)||
          url.getHost().equalsIgnoreCase(localHostIP)))
        {
          //...si manda al client la pagina HTML contenente l'applet
          //per gestire il proxy
          sendAdminPage();
        }
        //se  una richiesta delle classi applet del proxy...
        else if ((url.getHost().equalsIgnoreCase(localHostName)||
          url.getHost().equalsIgnoreCase(localHostIP)))
        {
          //...si invia l'applet richiesto
          sendAppletClass(url.getFile());
        }
        else 
        // una richiesta ad un web-server
        {
          hostIndex=config.isInHostList(url.getHost());
          //se  un sito da monitorare...
          if (hostIndex>-1)
          {
            body=new HtmlResponseBody(config,url.toString(),hostIndex);
            docIndex=-1;
            // ...se non  un sito gi sotto osservazione...
            if (!config.isObserved(hostIndex))
            {
              //...si invia la pagina contenente
              //l'applet per il riconoscimento dell'utente
              //N.B. in questo modo si perde la richiesta 
              //fatta dal client
              sendUserInfoPage(hostIndex,request);
              config.setLastSession(hostIndex);
              config.setIsObserved(hostIndex,true);
              isLastMessage=true;
            }
            //sito gi sotto monitoraggio
            //si pu inviare la richiesta del client
            else
            {
              //se  una query...
              if ((query=getQuery(url.toString())).length()>0)
              {
                //...se il messaggio non  codificato...
                if(!request.getIsMessageEncoded())
                {
                  //...prelevare le parole chiave
                  queryHandler(query);
                  //e apportare modifiche al DB Utenti
                }
              }
              //non  una query
              //se  la richiesta di un documento...
              else if(request.getMethod().equalsIgnoreCase("GET"))
              {
                //...si controlla il file richiesto
                docIndex=config.isInDocList(url.toString(),hostIndex);
                //se  un documento gi richiesto...
                if (docIndex>-1)
                {
                  //...se si sta tornando indietro da un link...
                  if (docIndex==config.getLastFatherDoc(hostIndex))
                    //...si apportano le relative modifiche 
                    //alla struttura dati della sessione attuale
                    config.updateInterestFactor(hostIndex,docIndex,2);
                  else
                    config.updateInterestFactor(hostIndex,docIndex,1);
                }
                else
                  //altrimenti si aggiorna la struttura 
                  //dati della sessione attuale
                  docIndex=config.insertIntoDocList(url.toString(),hostIndex);              
                // se  un link del documento precedente...
                linkIndex=config.isInLinkList(url.toString(),hostIndex);
                if (linkIndex>-1)
                {
                  // ...si apportano le relative modifiche al DB Utenti
                  //si aggiorna la struttura dati della sessione attuale
                  config.updateLinkList(linkIndex,hostIndex);
                  config.setLastFatherDoc(config.getLinkFatherDoc(hostIndex),hostIndex);
                }
                else
                  //si aggiorna la struttura dati della sessione attuale
                  config.setLastFatherDoc(-1,hostIndex);
              }
              //se si pu forwardare la richiesta...
              if (request.getIsToForward())
              {
                //...si invia la richiesta al proxy padre o al web-server
                //se non  gi stato creato...
                if (!isServerSocketAlive)
                  //...si crea il socket verso il web-server (o verso il proxy padre)
                  if (config.getIsFatherProxy())
                    serverSocket=new Socket(config.getFatherProxyHost(),
                                            config.getFatherProxyPort());
                  else
                  {
                    serverName=url.getHost();
                    serverSocket=new Socket(serverName(request.getUrl()),
                                            serverPort(request.getUrl()));
                    request.setUrl(serverUrl(request.getUrl()));
                  }
                serverOut=serverSocket.getOutputStream();
                serverDataOut=new DataOutputStream(serverOut);
                serverDataOut.writeBytes(request.toString(true));
                //serverDataOut.writeBytes(restOfRequestMessage(request,clientIn));
                serverDataOut.flush();
                sendRestOfRequestMessage(request,clientIn,serverOut);
                //gestione risposta dal web-server
                serverIn=serverSocket.getInputStream();
                //parsing dell'header della risposta del server
                response=new HttpResponseHdr();
                response.parse(serverIn);
                if (!isLastMessage)
                  isLastMessage=response.getIsLastMessage();
                //se il message body non  vuoto ed  completo...
                if ((response.getStatusCode().indexOf("200")>-1)||
                     (response.getStatusCode().indexOf("304")>-1))
                {
                  //...se  la risposta ad una GET...
                  if (request.getMethod().equalsIgnoreCase("GET"))
                  {
                    //se non  codificato...
                    if (!response.getIsMessageEncoded())
                    {
                      //...se  un file HTML...
                      if ((response.getContentType().toLowerCase().indexOf("text/html"))>-1)
                      {
                        //Si salva la vecchia lista dei link 
                        //relativa al vecchio actualHtmlDoc
                        config.saveLinkList(hostIndex,docIndex);
                        //se non lo abbiamo gi fatto...
                        if (!config.isParsed(hostIndex,docIndex))
                        {
                          //...Parsing del file alla ricerca dei link
                          // con cui si aggiorna la struttura 
                          //dati della sessione attuale
                          String bodyMess=getRestOfResponseMessage(serverIn);
                          body.parse(bodyMess);
                        }
                        //altrimenti abbiamo gi in memoria la lista 
                        //dei link presenti nel documento
                        else
                        {
                          config.loadLinkList(hostIndex,docIndex);
                          body.setMessageBody(getRestOfResponseMessage(serverIn));
                        }
                        //inserire un applet per contare il tempo di lettura
                        //e per aggiornare le strutture dati della sessione attuale
                        int newLength=body.insertApplet(hostIndex,docIndex,
                                                        config.getLocalHost(),
                                                        config.getAppletUrl(hostIndex),
                                                        config.getAdminPort());
                        response.setContentLength(newLength);
                        //N.B. se il file viene monitorato dall'applet
                        //si invia intero al client
                        htmlMonitor=true;
                        //...si invia al client tutto 
                        //il messaggio ricevuto dal server
                        clientDataOut=new DataOutputStream(clientOut);
                        clientDataOut.writeBytes(response.toString(true));
                        //resto del messaggio (si trova nell'html body)
                        clientDataOut.writeBytes(body.getMessageBody());
                        clientDataOut.flush();
                      }
                      //il documento non  un HTML
                      else
                      {
                        //si invia al client il messaggio 
                        //cos come  arrivato dal server
                        clientDataOut=new DataOutputStream(clientOut);
                        clientDataOut.writeBytes(response.toString(false));
                        clientDataOut.flush();
                        //resto del messaggio (si trova ancora nel serverIn)
                        sendRestOfResponseMessage(serverIn,clientOut);
                      }
                    }
                    //Il documento  codificato
                    else
                    {
                      //si invia al client il messaggio 
                      //cos come  arrivato dal server
                      clientDataOut=new DataOutputStream(clientOut);
                      clientDataOut.writeBytes(response.toString(false));
                      clientDataOut.flush();
                      //resto del messaggio (si trova ancora nel serverIn)
                      sendRestOfResponseMessage(serverIn,clientOut);
                    }
                  }
                  //richiesta diversa dalla GET
                  else
                  {
                    //si invia al client il messaggio 
                    //cos come  arrivato dal server
                    clientDataOut=new DataOutputStream(clientOut);
                    clientDataOut.writeBytes(response.toString(false));
                    clientDataOut.flush();
                    //resto del messaggio (si trova ancora nel serverIn)
                    sendRestOfResponseMessage(serverIn,clientOut);
                  }
                }
                //status code diverso da "200" o "304"
                //o messaggio vuoto
                else
                {
                  //si invia al client il messaggio 
                  //cos come  arrivato dal server
                  clientDataOut=new DataOutputStream(clientOut);
                  clientDataOut.writeBytes(response.toString(false));
                  clientDataOut.flush();
                  //resto del messaggio (si trova ancora nel serverIn)
                  sendRestOfResponseMessage(serverIn,clientOut);
                }
              }
              //la richiesta non deve essere passata 
              //al proxy successivo o al webserver
              else
              {
                //bisogna creare una risposta per il client
                sendOkMessage(request,clientIn,clientOut);
                isLastMessage=true;
              }
              if (isLastMessage)
              {
                //chiudere il socket verso il web-server
                serverSocket.close();
              }
            }
          }
          //non  un sito di nostro interesse
          else 
          {
            //se  la richiesta di un documento...
            if(request.getMethod().equalsIgnoreCase("GET"))
            {
              //controllo se  un link di qualche documento
              //degli host monitorati
              for (int k=0; k<config.getNumHost(); k++)
              {
                //se l'host  sotto osservazione
                if (config.isObserved(k))
                {
                  // se  un link dell'ultimo documento dell'host k...
                  linkIndex=config.isInLinkList(url.toString(),k);
                  if (linkIndex>-1)
                  {
                    //si aggiorna la struttura dati della sessione attuale
                    config.updateLinkList(linkIndex,k);
                    config.setLastFatherDoc(config.getLinkFatherDoc(k),k);
                  }
                }
              }
            }
            //se lo si pu forwardare...
            if (request.getIsToForward())
            {
              //...si invia la richiesta al proxy padre o al web server
              //se non  gi stato creato...
              if (!isServerSocketAlive)
                //...si crea il socket verso il web-server
                if (config.getIsFatherProxy())
                  serverSocket=new Socket(config.getFatherProxyHost(),
                              config.getFatherProxyPort());
                else
                {
                  serverName=url.getHost();
                  serverSocket=new Socket(serverName(request.getUrl()),
                              serverPort(request.getUrl()));
                }
              serverOut=serverSocket.getOutputStream();
              serverDataOut=new DataOutputStream(serverOut);
              serverDataOut.writeBytes(request.toString(false));
              serverDataOut.flush();
              if (request.getContentLength()>0)
                sendRestOfRequestMessage(request,clientIn,serverOut);
              //gestione risposta dal web-server
              serverIn=serverSocket.getInputStream();
              response=new HttpResponseHdr();
              response.parse(serverIn);
              if (!isLastMessage)
                isLastMessage=response.getIsLastMessage();
              //invio del documento al client
              clientDataOut=new DataOutputStream(clientOut);
              clientDataOut.writeBytes(response.toString(false));
              clientDataOut.flush();
              //resto del messaggio (si trova ancora nel serverIn)
              sendRestOfResponseMessage(serverIn,clientOut);
            }   
            //la richiesta non deve essere passata 
            //al proxy successivo o al webserver
            else
            {
              //bisogna creare una risposta per il client
              sendOkMessage(request,clientIn,clientOut);
              isLastMessage=true;
            }
            if (isLastMessage)
            {
              //chiudere il socket verso il web-server
              serverSocket.close();
            }
          }
        }
      }// Fine do
      //....fino a che connessione aperta
      while (!isLastMessage);      
      //chiudere il client socket
      clientSocket.close();
    }//Fine try
    catch(Exception e)
    {}
    finally
    {
      config.closeThread();
    }
  }// Fine run
  
  //Trova il //server.name da una url.
  //ritorna Servers internet name
  private String serverName(String str)
  {
    // riduce la stringa a "server.name:x/thing"
    int i=str.indexOf("//");   
    if (i<0)
      return "";  
    str=str.substring(i+2);
    // riduce la stringa a "server.name:xx"
    i=str.indexOf("/");
    if (0<i)
      str=str.substring(0,i);
    // riduce la stringa a "server.name"
    i=str.indexOf(":");
    if (0<i)
      str=str.substring(0,i);
    return str;  
  }//Fine serverName 
   
  //Trova la :PORT da "http://server.ect:PORT/some/file.xxx"
  //ritorna Servers internet port
  private int serverPort(String str)
  {
    // riduce la stringa a "server.name:x/thing"
    int i=str.indexOf("//");   
    if (i<0)
      return 80;  
    str=str.substring(i+2);
    // riduce la stringa a "server.name:xx"
    i=str.indexOf("/");
    if (0<i)
      str=str.substring(0,i);
    // riduce a "xx"
    i=str.indexOf(":");
    if (0<i)
    {
      return Integer.parseInt(str.substring(i).trim());
    }
    return 80;  
  }//Fine serverPort

  //Trova "/some/file.xxxx" da "http://server.ect:PORT/some/file.xxx"
  //ritorna la url
  private String serverUrl(String str)
  {
    int i= str.indexOf("//");   
    if (i<0)
      return str;   
    str=str.substring(i+2);
    i=str.indexOf("/");   
    if (i<0)
      return str;  
    return str.substring(i);   
  }//Fine serverUrl
  
  private void sendRestOfRequestMessage(HttpRequestHdr req,InputStream input,OutputStream output)
  {
    try
    {
      for (int i=0;i<req.getContentLength();i++)
        output.write(input.read());
      output.flush();
    }
    catch (IOException ioe)
    {}
  }//Fine sendRestOfRequestMessage
  
  private void sendRestOfResponseMessage(InputStream input, OutputStream output)
  {
    byte data[]= new byte[2000];
    int count;
    try
    {
      while ((count=input.read(data))>0)
      {
        output.write(data,0,count);
      }
      output.flush();
    }
    catch (IOException ioe)
    {}
  }//Fine sendRestOfResponseMessage
 
  private String getRestOfResponseMessage(InputStream input)
  {
    byte data[]= new byte[2000];
    int count;
    try
    {
      String mess="";
      while ((count=input.read(data))>0)
      {
        mess+=new String(data);
      }
      return mess;
    }
    catch (IOException ioe)
    {
      return "";
    }
 }//Fine getRestOfResponseMessage

  //invia la pagina HTML che contiene 
  //l'applet per amministrare il proxy
  private void sendAdminPage()
  {
    HttpReplyHdr reply=new HttpReplyHdr();
    String page="";
    String adminHost=config.getLocalHost();
    DataInputStream in;
    try
    {
      //File adminPage=new File(adminPath+File.separator+"Admin.html");
      File adminPage=new File(adminPath+"Admin.html");
      in=new DataInputStream(new FileInputStream(adminPage));
      String s=null;
      while((s=in.readLine())!=null)
        page+=s;
      page=page.substring(0,page.indexOf("PORT"))+
                          config.getAdminPort()+
                          page.substring(page.indexOf("PORT")+4);
      page=page.substring(0,page.indexOf("HOST"))+
                          adminHost+
                          page.substring(page.indexOf("HOST")+4);
      in.close();
      DataOutputStream out=new DataOutputStream(clientSocket.getOutputStream());
      out.writeBytes(reply.formOk("text/html",page.length(),false));
      out.writeBytes(page);
      out.flush();
      out.close();
    }
    catch (Exception e){
      System.out.println("Error: can't open admin page "+e.toString());
    }
  }//Fine sendAdminPage
  
  //invia il file richiesto
  private void sendAppletClass(String fileName)
  {
    try{
      int count;
      HttpReplyHdr reply=new HttpReplyHdr();
      if (fileName.startsWith("/")||
          fileName.startsWith("\\"))
          fileName=fileName.substring(1);
      File appletFile= new File(adminPath+fileName);
      long length=appletFile.length();
      byte data[]=new byte[2000];
      FileInputStream in=new FileInputStream(appletFile);
      DataOutputStream out=new DataOutputStream(clientSocket.getOutputStream());
      out.writeBytes(reply.formOk("application/octet-stream",length,false));
      while ((count=in.read(data))>-1)
      {
        out.write(data,0,count);
      }
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e) 
    {
      System.out.println(e.toString());
    }
  }//Fine sendAppletClass
  
  //invia la pagina HTML che contiene
  //l'applet per riconoscere l'utente
  private void sendUserInfoPage(int host, HttpRequestHdr req)
  {
    HttpReplyHdr reply=new HttpReplyHdr();
    String page="";
    String appletUrl=config.getAppletUrl(host);
    String dbHost=config.getHost(host);
    int dbPort=config.getDbPort(host);
    String adminHost=config.getLocalHost();
    int adminPort=config.getAdminPort();
    DataInputStream in;
    try{
      File userInfoPage=new File(adminPath+
                                 File.separator+
                                 "UserInfo.html");
      in=new DataInputStream(new FileInputStream(userInfoPage));
      String s=null;
      while((s=in.readLine())!=null)
        page+=s;
      page=page.substring(0,page.indexOf("APPLETURL"))
           +appletUrl
           +page.substring(page.indexOf("APPLETURL")+9);
      page=page.substring(0,page.indexOf("DBHOST"))
           +dbHost
           +page.substring(page.indexOf("DBHOST")+6);
      page=page.substring(0,page.indexOf("DBPORT"))
           +dbPort
           +page.substring(page.indexOf("DBPORT")+6);
      page=page.substring(0,page.indexOf("HOST"))
           +adminHost
           +page.substring(page.indexOf("HOST")+4);
      page=page.substring(0,page.indexOf("PORT"))
           +adminPort
           +page.substring(page.indexOf("PORT")+4);
      //se la richiesta del client era una GET
      if (req.getMethod().equalsIgnoreCase("GET"))
      {
        String richiesta=req.getUrl();
        //si inserisce un link alla url richiesta 
        //nella pagina UserInfo.html
        page=page.substring(0,page.indexOf("</APPLET>")+9)
             +"<BR> Per continuare la navigazione nel sito clicca <A href="
             +richiesta
             +">qui "
             +page.substring(page.indexOf("</APPLET>")+9);
      }
      in.close();
      DataOutputStream out=new DataOutputStream(clientSocket.getOutputStream());
      out.writeBytes(reply.formOk("text/html",page.length(),true));
      out.writeBytes(page);
      out.flush();
      out.close();
    }
    catch (Exception e){
      System.out.println("Error: can't open user info page");
    }
  }//Fine sendUserInfoPage
  
  private void sendOkMessage(HttpRequestHdr req, InputStream in, OutputStream out)
  {
    HttpReplyHdr reply=new HttpReplyHdr();
    String messageBody=new String();
    messageBody+=req.toString(false);
    long messageLength=messageBody.length();
    DataOutputStream dout=new DataOutputStream(out);
    try{
      dout.writeBytes(reply.formOk("message/http",messageLength,true));
      dout.writeBytes(messageBody);
      dout.flush();
      sendRestOfRequestMessage(req,in,out);
    }
    catch (IOException ioe){}
  }//Fine sendOkMessage
  
  private String getQuery(String url)
  {
    //Url=<scheme>://<net_loc>/<path>;<params>?<query>#<fragment>
    int index;
    //si elimina il <fragment>
    if ((index=url.indexOf("#"))>-1)
      url=url.substring(0,index);
    //si elimina lo <scheme>
    if ((index=url.indexOf(":"))>-1)
      url=url.substring(index+1);
    //si elimina il <net_loc>
    if (url.substring(0,2).equals("//"))
      url=url.substring(2);
    if ((index=url.indexOf("/"))>-1)
      url=url.substring(index+1);
    //si controlla se  presente la query
    if ((index=url.indexOf("?"))>-1)
      return url.substring(index);
    return "";
  }//Fine isAQuery
  
  private void queryHandler(String query)
  {
    //Sviluppi futuri
  }//Fine queryHandler
  
}//Fine classe Proxy
