package PseudoProxy;

import java.net.*;
import java.io.*;
import java.lang.*;

public class ProxyServer extends Thread
{
  //Member variables
  static ServerSocket mainSocket=null;
  static Config config;
  static File configFile;
  static int proxyServerPort;
  static boolean endProxy=false;
  final static int defaultProxyServerPort=8080;
  final static int maxProxyServerPort=65536;

  public ProxyServer(String args[])
  {
    
    //parse command line
    switch (args.length)
    {
      case 0:
        proxyServerPort=defaultProxyServerPort;
        break;
      case 1:
        try
        {
          proxyServerPort=Integer.parseInt(args[0]);            
        }
        catch (NumberFormatException nfe){
          System.out.println("Error: Invalid port number "+nfe);
          return;
        }
        if (proxyServerPort>maxProxyServerPort)
        {
          System.out.println("Error: Invalid port number");
          return;
        }
        break;
      default:
        System.out.println("Usage: Proxy [port number]");
        return;
    }//Fine switch
  }//Fine costruttore
    
  public void run()
  {
    DaemonThread daemonThd=null;
    try
    {
      // Crea una finestra per gestire il ProxyServer
      
      // Crea e inizializza il Config 
      // per configurare il proxy
      config=new Config();
      config.setServerAlive(true);
      config.setIsAppletContext(false);
      config.setLocalHost(InetAddress.getLocalHost().getHostName());
      String tmp=InetAddress.getLocalHost().toString();
      config.setLocalIP(tmp.substring(tmp.indexOf('/')+1));
      File adminDir=new File("");
      config.setAdminPath(adminDir.getAbsolutePath());
      //si crea il thread adminServer
      AdminServer adminSrv=new AdminServer(config);
      adminSrv.start();
      //Si caricano i parametri salvati della config
      configFile=new File("Config.txt");
      if (configFile.exists())
      {
        long length=configFile.length();
        if (length>0)
        {
          byte data[]=new byte[(int)length];
          FileInputStream inputFile=new FileInputStream(configFile);
          inputFile.read(data,0,(int)length);
          String configParameter=new String(data);
          if (configParameter.length()>0)
            config.parse(configParameter);
          inputFile.close();
        }
      }
      //Si crea il daemon-thread
      daemonThd=new DaemonThread(config);
      daemonThd.start();            
      // Creare il socket principale
      mainSocket=new ServerSocket(proxyServerPort);
      System.out.println("Proxy attivo...");
      //Main loop
      while(true)
      {
        // Si ascolta sul socket
        Socket clientSocket=mainSocket.accept();
        //Si passa la richiesta ad un nuovo proxy thread
        Proxy thd =new Proxy(clientSocket,config);
        thd.start();
      }
    }
    catch (Exception e)
    {
      System.out.println(e.toString());
    }
    finally
    {
      //si aspetta che tutti i proxy-thread terminino
      System.out.println("Attesa...");
      while(config.getThreads()>0)
      {}
      config.setServerAlive(false);
      System.out.println("Salvataggio informazioni...e chiusura ProxyServer");
      try 
      {
        mainSocket.close();
        configFile=new File("Config.txt");
        FileOutputStream outputFile=new FileOutputStream(configFile);
        outputFile.write(config.toString().getBytes());
        outputFile.close();
        System.out.println("Salvataggio dati nel DB Utenti");
        daemonThd.interrupt();
        daemonThd.join();
      }
      catch (Exception e)
      {
        System.out.println(e.toString());
      }
    }
  }//Fine run
  
  public void stopProxy()
  {
    try
    {
      mainSocket.close();
    }
    catch(Exception e)
    {}
  }
  
}// Fine classe ProxyServer
