import java.io.*;
import java.net.*;

public class ThreeTierHandler extends Thread
{
  private Socket client;
  private TwoTierOperator t2toper;
  
  public ThreeTierHandler(Socket cl)
  {
    super();
    client=cl;
    t2toper=new TwoTierOperator("jdbc:odbc:DBUtenti2");
  }//Fine costruttore
  
  public void run()
  {
    DataOutputStream out;
    BufferedReader in;
    int opCode;
    try
    {
      out=new DataOutputStream(client.getOutputStream());
      in=new BufferedReader(new InputStreamReader(client.getInputStream()));
      opCode=Integer.parseInt(in.readLine());
      switch (opCode)
      {
        //Operazione richiesta: insertNewUser
        case 1:
          insertNewUser(in,out);
          break;
        //Operazione richiesta: getUserAt
        case 2:
          getUserAt(in,out);
          break;
        //Operazione richiesta: getArgument
        case 3:
          getArgument(in,out);
          break;
        //Operazione richiesta: updateUserData
        case 4:
          updateUserData(in,out);
          break;
        //Operazione richiesta: updateUserInterest
        case 5:
          updateUserInterest(in,out);
          break;
        //Operazione richiesta: updateAverageUser
        case 6:
          updateAverageUser(in,out);
          break;
      }//Fine switch
    }
    catch (Exception e)
    {}
  }//Fine run
  
  private void insertNewUser(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String password=in.readLine();
      String name=in.readLine();
      String surname=in.readLine();
      String category=in.readLine();
      int age=Integer.parseInt(in.readLine());
      String login=t2toper.insertNewUser(password,
                                         name,
                                         surname,
                                         category,
                                         age);
      if (login.length()>0)
        out.writeBytes(login+"\n");
      else
        out.writeBytes("NULL\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine insertNewUser
  
  private void getUserAt(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String login=in.readLine();
      String password=in.readLine();
      boolean loginOK=t2toper.getUserAt(login,
                                        password);
      if (loginOK)
        out.writeBytes("TRUE\n");
      else
        out.writeBytes("FALSE\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine getUserAt
  
  private void getArgument(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String document=in.readLine();
      int argument=t2toper.getArgument(document);
      out.writeBytes(argument+"\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine getArgument
  
  private void updateUserData(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String login=in.readLine();
      String password=in.readLine();
      long lastSession=Long.parseLong(in.readLine());
      int argument=Integer.parseInt(in.readLine());
      boolean allRight=t2toper.updateUserData(login,
                                              password,
                                              lastSession,
                                              argument);
      if (allRight)
        out.writeBytes("TRUE\n");
      else
        out.writeBytes("FALSE\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine updateUserData
  
  private void updateUserInterest(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String login=in.readLine();
      String document=in.readLine();
      float value=Float.valueOf(in.readLine()).floatValue();
      boolean allRight=t2toper.updateUserInterest(login,
                                                  document,
                                                  value);
      if (allRight)
        out.writeBytes("TRUE\n");
      else
        out.writeBytes("FALSE\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine updateUserInterest
  
  private void updateAverageUser(BufferedReader in, DataOutputStream out)
  {
    try
    {
      String login=in.readLine();
      boolean allRight=t2toper.updateAverageUser(login);
      if (allRight)
        out.writeBytes("TRUE\n");
      else
        out.writeBytes("FALSE\n");
      out.flush();
      in.close();
      out.close();
    }
    catch (Exception e)
    {}
  }//Fine updateAverageUser
  
}//Fine ThreeTierHandler